
/**
 * @list dependencies
 */

var fs = require('fs')
  , url = require('url')
  , path = require('path')
  , util = require('util')
  , gzippo = require('gzippo')
  , express = require('express')
  , engine = require('ejs-locals')
  , promise = require('node-promise')
  , sessionStore = require('connect-mongoose')(express)

/**
 * @description single global, alias for app/express server
 */

global.app = express();

/**
 * @description load config
 */

app.config = require('./config/config')();

/**
 * @description setup and connect to mongodb
 */

app.mongo = require('mongoose');

app.mongo.connect(app.config.mongodb).connection.on('open', function() {
  console.log('> mongodb, connected!');
  app.mongo.set('debug', true);
}).on('error', function(error) {
  console.error({ mongodb : app.config.mongodb });
  console.error(error);
});

/**
 * @description configure express
 */

app.configure(function() {
  app.use(express.logger('tiny'));
  app.use(gzippo.staticGzip(__dirname + '/public'));
  app.use(express.bodyParser());
  app.use(express.cookieParser());
  app.use(express.session({
    secret : 'CHANGE_ME',
    maxAge : new Date(Date.now() + 3600000),
    store  : new sessionStore()
  }));
  app.engine('ejs', engine);
  app.set('views', __dirname + '/views');
  app.set('view engine', 'ejs');
});

/**
 * @description alias promise constructor and promise lib to app
 */

app.Promise = promise.Promise;
app.promise = promise;

/**
 * @description setup app structure
 */

app.schemas = {};
app.helpers = {};
app.services = {};
app.controllers = {};

/**
 * @description auto-load applicaton components
 */

(function(directories) {
  directories.map(function(directory) {
    var files = fs.readdirSync('./' + directory);
    files.sort(function(a, b) {
      return a < b;
    });
    files.map(function(file) {
      if (/\.js$/.test(file) && !/prototype/.test(file)) {
        var name = file.toLowerCase();
        var required = './' + directory + '/' + file;
        var type = directory.substr(0, directory.length - 1);
        var item = name.replace(type + '.js', '');
        try {
          app[directory][item] = require(required);
        } catch(error) {
          console.error('name:', name);
          console.error('item:', item);
          console.error('required:', required);
          console.error(error);
          throw new Error(error);
        };
      };
    });
  });
})(['helpers','schemas','services','controllers']);

/**
 * @description listen on port named in config
 */

app.listen(app.config.port, function() {
  console.log(util.format('> site running on port: %d', app.config.port));
});
