
/**
 * @prototype Model
 * @description All models are protypes of this class
 */

exports.Model = function(mongooseModel) {
  this.baseModel = mongooseModel;
};

/**
 * @method create
 */

exports.Model.prototype.create = function create(document) {
  var that = this
    , promise = new app.Promise();
  this.baseModel.create(document, function(error, data) {
    if (error) {
      if (error.message && error.message.match(/E11000/i)) {
        var errorReturn = {
          code    : 409,
          error   : error,
          message : 'Duplicate key error'
        };
      } else {
        var errorReturn = {
          code    : 400,
          error   : error,
          message : 'Validation error'
        };
      };
      promise.reject(errorReturn);
    } else {
      promise.resolve(data);
    };
  });
  return promise;
};

/**
 * @method find
 */

exports.Model.prototype.find = function find(query, fields, options) {
  var promise = new app.Promise();
  this.baseModel.find(query, fields, options, function(error, data) {
    if (error) {
      promise.reject({
        code    : 500,
        args    : query,
        error   : error,
        message : error.message
      });
    } else {
      promise.resolve(data);
    };
  });
  return promise; 
};

/**
 * @method fineOne
 */

exports.Model.prototype.findOne = function findOne(query, fields, options) {
  var promise = new app.Promise();
  this.baseModel.findOne(query, fields, options, function(error, data) {
    if (error) {
      promise.reject({
        code    : 500,
        args    : query,
        error   : error,
        message : error.message
      });
    } else {
      promise.resolve(data);
    };
  });
  return promise;
};

/**
 * @method findById
 */

exports.Model.prototype.findById = function findById(id, fields, options) {
  var promise = new app.Promise();
  this.baseModel.findById(id, fields, options, function(error, data) {
    if (error) {
      promise.reject({
        code    : 500,
        args    : id,
        error   : error,
        message : error.message
      });
    } else {
      promise.resolve(data);
    };
  });
  return promise;
};

/**
 * @method remove
 */

exports.Model.prototype.remove = function remove(conditions) {
  var promise = new app.Promise();
  this.baseModel.remove(conditions, function(error) {
    if (error) {
      promise.reject({
        code: 500,
        error: error,
        message: 'Invalid arguments'
      });
    } else {
      promise.resolve(conditions);
    };
  });
  return promise;
};

/**
 * @method update
 */

exports.Model.prototype.update = function update(query, document, options) {
  var that = this
    , promise = new app.Promise()
    , args = Array.prototype.slice.call(arguments, 0);
  this.baseModel.update(query, document, function(error, affected) {
    if (error) {
      promise.reject(error, true);
    } else {
      if (affected === 0) {
        promise.reject({
          code    : 500,
          message : 'MongoDB - Cannot find Document'
        });
      } else {
        promise.resolve();
      };
    };
  });
  return promise;
};

/**
 * @method count
 */

exports.Model.prototype.count = function count(query) {
  query = query || {};
  var promise = new app.Promise();
  this.baseModel.count(query, function(error, data) {
    if (error) {
      promise.reject({
        code    : 500,
        args    : query,
        error   : error,
        message : 'MongoDB Count Error'
      }, true);
    } else {
      promise.resolve(data);
    };
  });
  return promise;
};
