
/**
 * @service User
 * @description User related Services? :P
 */

var User = app.schemas.user.User;

/**
 * @method setSession
 * @param {Object} request
 * @param {Object} user user mongo object
 * @description sets session data for users and finds role
 */

exports.setSession = function setSession(params) {
  var user = params.user
    , request = params.request
    , promise = app.Promise();
  if (user.password) {
    delete user.password;
  };
  try {
    request.session.auth = {
      logged : true,
      user   : user
    };
    promise.resolve(user);
  } catch (error) {
    promise.reject(error, true);
  };
  return promise;
};

/**
 * @method login
 * @param {Object} data contains email and password fields
 * @description tries to login user and set their session
 */

exports.login = function(request, data) {
  var promise = new app.Promise();
  var query = { 
    email    : data.email, 
    password : app.helpers.general.encryptPass(data.password)
  };
  var findOnePromise = User.findOne(query);
  findOnePromise.then(function(user) {
    if (user !== null && user._id) {
      var sessionPromise = app.services.user.setSession({
        user    : user,
        request : request
      });
      sessionPromise.then(function(user) {
        promise.resolve(user);
      }, function(error) {
        promise.reject(error, true);
      });
    } else { 
      promise.reject({
        message : 'Invalid Login',
        code    : 403
      }, true);
    };
  }, function(error) {
    promise.reject(error, true);
  });
  return promise;
};

/**
 * @method createUser
 */

exports.createUser = function createUser(request, data) {
  var promise = new app.Promise()
    , user = data
    , insertPromise = User.create(user);
  insertPromise.then(function(result) {
    promise.resolve({
      user    : result,
      request : request
    });
  }, function(error) {
    promise.reject(error, true);
  });
  return promise;
};

/**
 * @method register
 */

exports.register = function register(request, data) {
  var promise = new app.Promise();
  var actions = [];
  // 1. create a user.
  actions.push(function() {
    return app.services.user.createUser(request, data);
  });
  // 2. set session
  actions.push(app.services.user.setSession);
  // iterate over actions sequentially
  app.promise.seq(actions).then(function(user) {
    // remove password
    delete user.password;
    console.log('> created user: ' + user.email);
    promise.resolve(user);
  }, function(error) {
    console.error(error);
    promise.reject(error, true);
  });
  return promise;
};
